#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@compose-x.io>

from ecs_composex import __version__ as version

metadata = {
    "Type": "ComposeX",
    "Properties": {
        "ecs_composex::module": "ecs_composex.kms",
        "Version": version,
    },
}
