#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@compose-x.io>

"""
Module to handle ECS services access.
"""

ACCESS_TYPES = {"Publish": {"Action": ["sns:Publish"], "Effect": "Allow"}}
