#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@compose-x.io>

"""
Global settings and variables re-used across the project
"""

XFILE_DEST = "ComposeXFile"
DIR_DEST = "OutputDirectory"
CFN_EXPORT_DELIMITER = r"::"
X_KEY = r"x-"
X_AWS_KEY = r"x-aws-"
